#############################################################################
##
#W   check.g               share package 'enumpgrp'              Bettina Eick
##
##   check routines for enumpgrp.

#############################################################################
##
#F TriangulizedMat( mat )
##
TriangulizedMat := function(mat)
    local new;
    new := ShallowCopy( mat );
    TriangulizeMat(new);
    return new;
end;

#############################################################################
##
#F CheckOper( mat, bases )
##
CheckOper := function( mat, sub )
    local f, bases, list, b, c;
    f := function( pt, mat ) return TriangulizedMat( pt * mat ); end;
    bases := List( sub, x -> TriangulizedMat( Basis( x ) ) );
    list := [];
    for b in bases do
        c := f( b, mat );
        Add( list, Position( bases, c ) );
    od;
    if Length( Set( list ) ) > Length( list ) then
        Error("objects are not permuted");
    fi;
end;

#############################################################################
##
#F CheckDescendants( G, M ) . . . . number of descendants from SmallGroups cat
##
CheckDescendants := function( G, M )
    local p, c, n, d, r, all, H, ser, F, N;
    p := PrimePGroup( G );
    c := PClassPGroup( G );
    n := Length( Pcgs(G) );
    d := M.dimension;
    r := 0;
    all := AllGroups( p^(d+n), PClassPGroup, c+1 );
    for H in all do
        ser := PCentralSeries( H, p );
        N := ser[Length(ser)-1];
        F := H / N;
        if Size(F) = Size(G) then
            if IdGroup(F) = IdGroup( G ) then
                r := r + 1;
            fi;
        fi;
    od;
    return r;
end;

#############################################################################
##
#F PCentrePGroup( H ) . . . . . . . . . . . last subgroup of p-central series
##
PCentrePGroup := function( H )
    local ser;
    ser := PCentralSeries(H, PrimePGroup( H ));
    return ser[Length(ser)-1];
end;

#############################################################################
##
#F DescendantOf( H ) . . . . . . . . . . . .find the id of the `parent' group
##
DescendantOf := function( H )
    return IdGroup( H/PCentrePGroup(H) );
end;

