#############################################################################
##
#F PCoverAgGroup( <G> ). . . . . . . . . . . . . . . . compute p-cover of <G>
##
PCoverAgGroup := function( G )

    local P, PQ, Gcgs, Ggens, F, Fgens, hom, def, Frels, Prels, i, j,
          w, v, f, n;

    Gcgs  := Cgs(G);
    n     := Length( Gcgs );
    G.prime := RelativeOrderAgWord( G.generators[1] );
    if not IsBound( G.series ) then
        G.series := G.operations.PCentralSeries( G, G.prime );
    fi;
    Ggens := FactorArg( G, G.series[2] ).generators;
    G.rang := Length( Ggens );
    G.field := GF( G.prime );
    G.pClass := Length( G.series ) - 1;

    # compute presenation on minimal generating set
    F     := FreeGroup( G.rang );
    Fgens := F.generators;
    hom   := GroupHomomorphismByImages( F, G, F.generators, Ggens );
    hom.isHomomorphism := true;
    hom.isSurjective   := true;

    # get definitions of cgs
    def := [];
    for i in [1..G.rang] do
        def[i] := F.generators[i];
    od;
    for i in [G.rang+1..n] do
        def[i] := hom.operations.PreImagesRepresentative( hom, Gcgs[i] );
    od;

    # compute relators for presentation
    Frels := [];
    for i in [1..n] do
        w := def[i]^G.prime;
        v := hom.operations.PreImagesRepresentative( hom, Gcgs[i]^G.prime );
        Add( Frels, w / v );
    od;
    for i in [1..n] do
        for j in [i+1..n] do
            w := Comm( def[j], def[i] );        
            v := Comm( Gcgs[j], Gcgs[i] );
            v := hom.operations.PreImagesRepresentative( hom, v );
            Add( Frels, w / v );
        od;
    od;

    # now compute relators for P
    Prels := [];
    for f in Fgens do
        Append( Prels, List( Frels, x -> Comm(x, f) ) );
    od;
    Append( Prels, List( Frels, x -> x^G.prime ) );

    F.relators := Prels;
    PQ := pQuotient( F, G.prime, G.pClass+1);

    # now set up P
    Unbind( PQ.agGroup );
    P := AgGroup( PQ );
    P.def := PQ.definedby;
    
    # add information
    P.rang   := G.rang;
    P.prime  := G.prime;
    P.field  := G.field;
    P.series := P.operations.PCentralSeries( P, P.prime );
    P.pClass := Length( P.series ) - 1;
    P.epi    := P.operations.GroupHomomorphismByImages( P, G, 
                P.generators{[1..P.rang]}, G.generators{[1..G.rang]});
    P.epi.isHomomorphism := true;
    P.epi.isSurjective   := true;
    return P;
end;
 
