#############################################################################
##
#W  createtab.gi           share package 'enumpgrp'              Bettina Eick
##

#############################################################################
##
#F CreateEnumerationTablePClass2 . . . . . . . . . . . .  groups of order p^l
##
CreateEnumerationTablePClass2 := function( p, l )
    local tab, gls, d, m, h, G, M, clA, clB, e, A, t, w, run;

    Info(InfoEnum, 1, "compute conjugacy classes ");
    t := Runtime();
    tab := List( [1..l-1], x -> [] );
    run := List( [1..l-1], x -> [] );
    gls := List( [1..l-1], x -> ConjugacyClasses( GL( x, p ) ) );
    tab[1][1] := 1;
    w := Runtime() - t;

    for d in [2..l-1] do
        m := Minimum( d*(d+1)/2, l-d );
        for h in [1..m] do
            if h = d*(d+1)/2 then
                tab[d][d+h-1] := 1;
            elif h = 1 then
                tab[d][d+h-1] := d + QuoInt(d,2);
            else
                Info(InfoEnum, 1, "enumeration of ",d," by ",h);
                w := Runtime();
                G := ElementaryAbelianGroup( p^d );
                A := AutomorphismGroup( G );
                M := TrivialGModuleOfGroup( G, h );
                clA := gls[d];
                clA := TranslateMatsToAutomorphisms( G, A, clA );
                clB := gls[h];
                e := EnumerateDescendantsWithClasses2( G, M, clA, clB );
                tab[d][d+h-1] := e;
                w := Runtime() - w;
                run[d][d+h-1] := w;
                Info( InfoEnum, 1, "\n");
            fi;
        od;
    od;
    t := Runtime() - t;
    return rec( table := tab, times := run, sum := StringTime(t),
                timeMatAct := StringTime( TMA ),
                timeIntVSp := StringTime( TVS ) );
end;

#############################################################################
##
#F CreateEnumerationTablePClass3  . . . . . . . . . . . . groups of order p^n
##
CreateEnumerationTablePClass3 := function( p, n )
    local tab, run, i, d, M, clB, c, all, nr, G, A, clA, e, r, j;

    # the stupid cases
    if n < 3 then return 0; fi;
    if n = 3 then return 1; fi;

    # set up
    tab := List( [1..n-1], x -> 0 );
    run := List( [1..n-1], x -> [] );

    for i in Reversed( [3..n-1] ) do
        d := n-i;
        M := TrivialGModuleOfGroup(ElementaryAbelianGroup(p^i), d);
        clB := ConjugacyClasses(GL(d, p));
        r := NumberSmallGroups( p^i );
        for j in [1..r] do
            G := SmallGroup( p^i, j );
            if IsCyclic( G ) then 
                if d = 1 then 
                    tab[i] := tab[i] + 1;
                fi;
            elif not IsElementaryAbelian(G) and Capability( G ) >= d then
                Info( InfoEnum, 1, "start group ",IdGroup(G));
                SetIsPGroup(G, true);
                A := AutomorphismGroup( G );
                clA := ConjugacyClassesAutGrp( A );
                e := EnumerateDescendantsWithClasses( G, M, clA, clB );
                tab[i] := tab[i] + e;
                Info( InfoEnum, 1, "\n");
            fi;
        od;
    od;
    return tab;
end;

