#############################################################################
##
#W  determine.gi           share package 'enumpgrp'              Bettina Eick
##

#############################################################################
##
#F ListDescendantsWithAutGrps( G, M, A, B )
##
ListDescendantsWithAutGrps := function( G, M, A, B )
    local cc, V, elms, sub, s, D, gens, mats, DD, res, e, o, f;

    # compute cohomology
    Info(InfoEnum, 2, "compute cohomology ");
    cc := TwoCohomology( G, M );
    V := Image( cc.cohom );
    elms := Elements( V );
    # Print(Length(elms), " elements remaining \n");

    # subtract the non-descendant subspaces
    Info(InfoEnum, 2, "compute non-descendant subspaces ");
    sub := NonDescendantSubspaces( G, M, cc );
    for s in sub do
        elms := Filtered( elms, x -> not x in s );
        # Print(Length(elms), " elements remaining \n");
    od;

    # compute matrix action on cc
    Info(InfoEnum, 2, "compute matrix action ");
    D := DirectProduct( A, B );
    gens := GeneratorsOfGroup(D);
    mats := MatrixOperationOfCPGroup( cc, gens );
    mats := List( mats, TransposedMat );
    DD   := Group( mats, mats[1]^0 );

    res := [];
    while Length( elms ) > 0 do

        # store element as code
        e := elms[1];
        f := PreImagesRepresentative( cc.cohom, e );
        f := ExtensionSQ( cc.collector, G, M, f );
        f := CodePcGroup( f );
        Add( res, f );
     
        # subtract orbit
        o := Orbit( DD, e, OnRight );
        elms := Filtered( elms, x -> not x in o );
        # Print(Length(elms), " elements remaining \n");
    od;
    return res;
end;

#############################################################################
##
#F ListDescendants( G, M ) 
##
ListDescendants := function( G, M )
    local A, B;
    A := AutomorphismGroup( G );
    B := GL( M.dimension, Characteristic( M.field ) );
    return ListDescendantsWithAutGrps( G, M, A, B );
end;

