#############################################################################
##
#W  general.gi             share package 'enumpgrp'              Bettina Eick
##

#############################################################################
##
#F TrivialGModuleOfGroup( G )
##
TrivialGModuleOfGroup := function( G, d )
    local pcgs, p, mats, M;
    pcgs := Pcgs(G);
    p := RelativeOrders( pcgs )[1];
    mats := List( pcgs, x -> IdentityMat( d, GF(p) ) );
    M := GModuleByMats( mats, GF(p) );
    M.isCentral := true;
    return M;
end;

#############################################################################
##
#F HighestWeight( G )
##
HighestWeight := function( G )
    local pcgs, sers, ind, dep, r;
    
    pcgs := Pcgs( G );
    sers := PCentralSeries( G, RelativeOrders(pcgs)[1] );
    ind  := InducedPcgs( Pcgs(G), sers[Length(sers)-1] );
    dep  := List( ind, x -> DepthOfPcElement( pcgs, x ) );
    r    := dep[1];
    if not dep = [r..Length(pcgs)] then 
        Error("pc presentation of G is not weighted"); 
    fi;
    return r;
end;

#############################################################################
##
#F MatToAutomorphism( pcgs, mat )
##
MatToAutomorphism := function( G, pcgs, mat )
    local  imgs, auto;
    imgs := List( [1..Length(pcgs)], 
                   x -> PcElementByExponents( pcgs, mat[x] ) );
    auto := GroupHomomorphismByImages( G, G, pcgs, imgs );
    return auto;
end;

#############################################################################
##
#F TranslateMatsToAutomorphism( G, A, clA )
##
TranslateMatsToAutomorphisms := function( G, A, clA )
    local pcgs, i, a, new, cl;
    pcgs := Pcgs( G );
    cl := [];
    for i in [1..Length(clA)] do
        a := MatToAutomorphism(G, pcgs, Representative(clA[i]));
        new := ConjugacyClass( A, a );
        SetSize(new, Size(clA[i]));
        Add( cl, new );
    od;
    return cl;
end;

#############################################################################
##
#F TranslatePermsToAutomorphisms( G, A, elm, cls )
##
TranslatePermsToAutomorphisms := function( G, A, elm, cls )
    local pcs, gen, new, cl, g, img, hom, tmp;
    pcs := Pcgs(G);
    gen := List(pcs, x -> Position(elm,x));
    new := [];
    for cl in cls do
        g := Representative(cl);
        img := elm{List( gen, x -> x^g )};
        hom := GroupHomomorphismByImagesNC( G, G, pcs, img ); 
        SetIsBijective( hom, true );
        tmp := ConjugacyClass( A, hom );
        SetSize( tmp, Size(cl ) );
        Add( new, tmp );
    od;
    return new;
end;

#############################################################################
##
#F ConjugacyClassesAutGrp( A )
##
ConjugacyClassesAutGrp := function( A )
    local G, pcs, cls, new, elm, P;

    # first check
    if HasConjugacyClasses( A ) then return ConjugacyClasses( A ); fi;

    # the class 2 case is easy
    G  := Source(Identity(A));
    if IsElementaryAbelian(G) then
        pcs := Pcgs(G);
        cls := ConjugacyClasses( GL( Length(pcs), PrimePGroup(G) ) );
        new := TranslateMatsToAutomorphisms(G, A, cls); 
    else
        elm := Elements(G);
        P   := Operation( A, elm );
        #cls := ConjugacyClassesPermGroup( P );
        cls := ConjugacyClasses( P );
        new := TranslatePermsToAutomorphisms( G, A, elm, cls );
    fi;
    SetConjugacyClasses( A, new );
    return new;
end;

#############################################################################
##
#F NonDescendantSubspaces( G, M, cc )
##
NonDescendantSubspaces := function( G, M, cc )
    local V, nor, maxs, subs, sub, pcgs, hom, co, r, m, b, c, i, j, tmp, base,
          full, zero, use;

    if IsElementaryAbelian(G) and M.dimension = 1 then return []; fi;

    V := M.field^M.dimension;
    nor := NormedVectors( V );
    maxs := List(nor, x -> OrthogonalSpaceInFullRowSpace(Subspace(V,[x])));
    hom := cc.cohom;
    co  := Source( hom );
    pcgs := Pcgs( cc.group );
    r := HighestWeight( G );
    zero := List( cc.presentation.relators, x -> Zero( V ) );
    full := IdentityMat( M.dimension, M.field );

    subs := [];
    for m in maxs do
        sub := [];
        base := Basis( m );
        for i in [1..Length(pcgs)] do
            for j in [1..i] do
                if i >= r then
                    use := base;
                else
                    use := full;
                fi;
                for b in use do
                    c := ShallowCopy( zero );
                    c[i*(i-1)/2 + j] := b;
                    Add( sub, Flat( c ) );
                od;
            od;
        od;
        sub := VectorSpace( M.field, sub, Flat(zero), "basis" );
        Add( subs, sub );
    od;

    if not IsElementaryAbelian(G) then
        for i in [1..Length(subs)] do
            subs[i] := IntersectionVS( Source(hom), subs[i] );
            subs[i] := Image( hom, subs[i] );
        od;
        subs := FindMaximals( subs );
    fi;
    return subs;
end;

