#############################################################################
##
#W  tensor.gi              share package 'enumpgrp'              Bettina Eick
##

#############################################################################
##
#F SizeOfFullUnion( p, d, l )
##
SizeOfFullUnion := function( p, d, l )
    local com, s, c, t, u, i;
    if d > l then return 0; fi;
    com := Combinations( [1..l], d );
    s := 0;
    for c in com do
        t := 1;
        u := 0;
        for i in [1..l] do
            if i in c then
               t := t * (p^d - p^u);
               u := u + 1;
            else
               t := t * p^u;
            fi;
        od;
        s := s + t;
    od;
    return s; 
end;

#############################################################################
##
#F BlockBasis( V, d, l ) . . . rewrite as matrix space
##
BlockBasis := function( V, d, l )
    local basis, index, i, mat, j;

    # first create basis of TP form
    basis := ShallowCopy( Basis( V ) );
    TriangulizeMat( basis );
    index := List( [1..l], x -> (x-1)*d );
    for i in [1..Length(basis)] do
        mat :=  [];
        for j in [1..d] do
            Add( mat, basis[i]{index+j} );
        od;
        basis[i] := mat;
    od;
    return basis;
end;

#############################################################################
##
#F TriangulizeTP
##
TriangulizeTP := function( base, d, l )
    local mat, i, j, mats, new;
    
    # paste matrices together
    mat := [];
    for i in [1..d] do
        mat[i] := [];
        for j in [1..Length(base)] do
            Append( mat[i], base[j][i] );
        od;
    od;

    # triangulize
    TriangulizeMat( mat );

    # cut into matrices
    mats := [];
    for i in [1..Length(base)] do
        new := [];
        for j in [1..d] do
            Add( new, mat[j]{[(i-1)*l+1..i*l]} );
        od;
        Add(mats, new );
    od;
    return mats; 
end;

#############################################################################
##
#F IsFlat( p, d, l, U )
##
IsFlat := function( p, d, l, U )
    local basis, mats;
    if Dimension(U) = 0 then return true; fi;
    basis := BlockBasis( U, d, l );
    mats := TriangulizeTP( basis, d, l );
    return ForAll( mats, x -> x[d] = 0 * x[d] );
end;

#############################################################################
##
#F IsFull( p, d, l, U )
##
IsFull := function( p, d, l, U )
    local basis, heads, pos, sub,index, symm, j, b, k, zero;
    basis := Basis( U );
    heads := 0;
    pos := 1;
    while pos <= Length (basis) do
        if Length( basis )< pos+d-1 then
            return false;
        fi;

        # check the next d basis elements
        sub := basis{[pos..pos+d-1]};
        pos := pos + d;
 
        # check whether sub is symmetric
        index := List( [1..l], x -> (x-1)*d );
        symm := sub[1]{index+1};
        zero := 0 * symm;
        for j in [1..d] do
            b := sub[j];
            for k in [1..d] do
                if k = j and not b{index+j} = symm then
                    return false;
                fi;
                if k <> j and b{index+k} <> zero then
                    return false;
                fi;
            od;
        od;

        # it is symmetric, add head
        heads := heads + 1;
    od;
    return heads;
end;
