#############################################################################
##
#W  union.gi               share package 'enumpgrp'              Bettina Eick
##

#############################################################################
##
#F Intersection of VS with time
##
IntersectionVS := function( V, W )
    local t, I, s, f, a, b;

    if Dimension(V) = 0 then return V; fi;
    if Dimension(W) = 0 then return W; fi;

    t := Runtime();
    I := Intersection( V, W );
    TVS := TVS + Runtime() - t;
    return I;
end;

#############################################################################
##
#F Matrix Oper of CP with time
##
MatrixOperationOfCPGroupWT := function( cc, gens )
    local t, r;
    t := Runtime();
    r := MatrixOperationOfCPGroup( cc, gens );
    TMA := TMA + Runtime() - t;
    return r;
end;

#############################################################################
##
#F FindMaximals( sub ) . . . . . . . .subgroups which are maximal within sub
##
FindMaximals := function( sub )
    local new, i;
    new := [];
    for i in [1..Length(sub)] do
        if Dimension( sub[i] ) > 0 then
            if not ForAny( new, x -> IsSubspace( x, sub[i] ) ) then
                new := Filtered( new, x -> not IsSubspace( sub[i], x ) );
                Add( new, sub[i] );
            fi;  
        fi;
    od;
    return new;
end;

#############################################################################
##
#F SizeOfUnionRec( sub ) -- recursive version
##
SizeOfUnionRec := function( list )
    local s, i, int, t, n;

    s := Size( list[1] );
    for i in [2..Length(list)] do
        int := List( list{[1..i-1]}, x -> IntersectionVS( x, list[i] ) );
        t := SizeOfUnion( int );
        s := s + Size( list[i] ) - t;
    od;
    return s;
end;

#############################################################################
##
#F SizeOfUnionRecDual( sub ) -- recursive version using splittings 
##
SizeOfUnionRecDual := function( list )
    local h, s1, s2, int, i, j, U, t, n;

    # split list and compute size of parts
    n := Length( list );
    h := QuoInt(n, 2);

    # first the intersection
    int := [];
    for i in [1..h] do
        for j in [h+1..n] do
            U := IntersectionVS( list[i], list[j] );
            Add( int, U );
        od;
    od;
    t := SizeOfUnion( int );

    # now the two parts
    s1 := SizeOfUnion( list{[1..h]} );
    s2 := SizeOfUnion( list{[h+1..n]} );
   
    return  s1 + s2 - t;
end;

#############################################################################
##
#F SizeOfUnionIter -- iterative version
##
SizeOfUnionIter := function( list )
    local field, r, com, int, s, i, c, l, j, u;
 
    # the trivial cases
    if Length( list ) = 0 then return 1; fi;
    if Length( list ) = 1 then return Size( list[1] ); fi;
    
    # set up                      
    r := Length( list );
    
    # inclusion - exclusion prinziple
    com := Combinations( [1..r] );
    com := com{[2..Length(com)]};
    int := List( com, x -> true );  
    s   := 0;                      
    for i in [1..Length( com )] do
        c := com[i];                   
        l := Length( c ); 
        if l = 1 then
            int[i] := list[c[l]];
            s := s + Size( int[i] );
        else                                                   
            u := c{[1..l-1]};
            j := Position( com, u );
            int[i] := IntersectionVS( int[j], list[c[l]] );
            s := s + (-1)^(l+1) * Size( int[i] );
        fi;
    od;                             
    return s;
end;

#############################################################################
##
#F SizeOfUnionTriv -- trivial version
##
SizeOfUnionTriv := function( list )
    return Length( Union( List(list, Elements) ) );
end;

#############################################################################
##
#F SizeOfUnion -- main function 
##
InstallGlobalFunction( SizeOfUnion,
function( sub )
    local list, s, t;
    list := FindMaximals( sub );
    if Length(list) = 0 then return 1; fi;
    if Length(list) = 1 then return Size(list[1]); fi;
    return SizeOfUnionRec( list );
end );
 
